/*
 * Direct3D HAL GLX interface
 *
 * Copyright 2003 TransGaming Technologies, Inc.
 */
#ifndef __WINE_GLX_PRIVATE_H
#define __WINE_GLX_PRIVATE_H
#include "config.h"
#ifdef HAVE_OPENGL
#include "ddrawi.h"

#define CACHESIZE 64 /* at least front + back + 6 cubemap faces = 8, preferably more */

typedef struct _D3DTCONTEXT {
  struct _D3DCONTEXT* parent;
  HANDLE	thread;
  GLXContext	ctx;
  GLXContext	save_ctx;
  GLXDrawable	save_draw;
  DWORD		stamp;
  struct _D3DTCONTEXT*	prev;
  struct _D3DTCONTEXT*	next;
} D3DTCONTEXT,*LPD3DTCONTEXT;

typedef struct {
  GLXDrawable draw;
  DWORD flags;
  HBITMAP bmp;
  X11DRV_DIBSECTION *dib;
  BITMAPOBJ *lockedBmp;
  DWORD locks, lockf;
  BOOL rect_lock;
  RECTL rect;
} D3DTARGET,*LPD3DTARGET;

typedef struct _D3DCONTEXT {
  D3DTCONTEXT	p;

  XVisualInfo*	vis;
  DWORD		owner_tid;
  pid_t		owner_unix_tid;
  GLXContext	owner_ctx;
  BOOL		borked, cant_copy;
  DWORD		refs;
  LPDDRAWI_DDRAWSURFACE_LCL lsurf, zsurf, nsurf;
  LPDDRAWI_DDRAWSURFACE_GBL surf;
  Font		lfont;

  D3DTARGET	cache[CACHESIZE];
  D3DTARGET	zcache[CACHESIZE];
  DWORD		curp;

  GLint		texunits;
  void (*glActiveTextureARB)(GLenum texture);

  int		glx_major, glx_minor;

  /* SGIX_fbconfig */
  int (*glXGetFBConfigAttribSGIX)(Display*,GLXFBConfig,int,int*);
  GLXFBConfig* (*glXChooseFBConfigSGIX)(Display*,int,const int*,int*);
  GLXPixmap (*glXCreateGLXPixmapWithConfigSGIX)(Display*,GLXFBConfig,Pixmap);
  GLXContext (*glXCreateContextWithConfigSGIX)(Display*,GLXFBConfig,int,GLXContext,Bool);
  XVisualInfo* (*glXGetVisualFromFBConfigSGIX)(Display*,GLXFBConfig);
  GLXFBConfig (*glXGetFBConfigFromVisualSGIX)(Display*,XVisualInfo*);

  /* SGIX_pbuffer */
  GLXPbuffer (*glXCreateGLXPbufferSGIX)(Display*,GLXFBConfig,unsigned int,unsigned int,int*);
  void (*glXDestroyGLXPbufferSGIX)(Display*,GLXPbuffer);
  void (*glXQueryGLXPbufferSGIX)(Display*,GLXPbuffer,int,unsigned int);
  void (*glXSelectEventSGIX)(Display*,GLXDrawable,unsigned long);
  void (*glXGetSelectedEventSGIX)(Display*,GLXDrawable,unsigned long*);

#ifdef TRANSGAMING_LOGO
  GLuint logo;
  GLuint logo_list;
  DWORD logo_size;
#endif
} D3DCONTEXT,*LPD3DCONTEXT;
typedef const D3DCONTEXT* LPCD3DCONTEXT;

#define FLAG_PIXMAP	0x0001
#define FLAG_CINDIRECT	0x0002
#define FLAG_PINDIRECT	0x0004
#define FLAG_WINDIRECT  0x0008
#define FLAG_PDIRECT	0x0010
#define FLAG_WDIRECT	0x0020
#define FLAG_INDIRECT	(FLAG_PINDIRECT|FLAG_WINDIRECT)
#define FLAG_DIRECT	(FLAG_PDIRECT|FLAG_WDIRECT)
#define FLAG_BACKBUF	0x0040
#define FLAG_FLIPPED	0x0080
#define FLAG_PBUFFER	0x0100
#define FLAG_OFFSCREEN	(FLAG_PIXMAP|FLAG_PBUFFER)
#define FLAG_ZBUFFER	0x0200
#define FLAG_DESTROYED	0x8000

#define FLAG_COMMON	(FLAG_INDIRECT|FLAG_DIRECT|FLAG_ZBUFFER|FLAG_DESTROYED)

DWORD X11DRV_GLX_MakeCurrent(LPD3DCONTEXT ctx);
void X11DRV_GLX_FlushCurrent(LPD3DCONTEXT ctx);
BOOL X11DRV_GLX_KillCurrent(LPD3DCONTEXT ctx);

extern void X11DRV_GLX_EB_Init(void);
extern void X11DRV_GLX_EB_Context_Init(void);
extern void X11DRV_GLX_EB_Cleanup(void);

extern DDHAL_DDEXEBUFCALLBACKS X11DRV_GLX_exebufcallbacks;
extern struct GL_D3D_DriverCallbacks X11DRV_GLX_GL_D3D_driver_callbacks;

extern BOOL X11DRV_GLX_D3dRenderLock(ULONG_PTR dwhContext, BOOL draw, LPBOOL invert);
extern void X11DRV_GLX_D3dRenderUnlock(ULONG_PTR dwhContext, BOOL draw);
extern void X11DRV_GLX_D3dRenderArea(ULONG_PTR dwhContext, LPRECT rect);

#endif /* HAVE_OPENGL */
#endif /* __WINE_GLX_PRIVATE_H */
